from contextlib import contextmanager
from docker.types import DeviceRequest
from logging import getLogger
import tarfile
from tarfile import TarFile, TarInfo
from tempfile import TemporaryFile
from time import time
from contextlib import closing
from io import BytesIO


LOGGER = getLogger(__name__)


@contextmanager
def create_temporary_docker_volume(client, *args, **kwargs):
    volume = client.volumes.create(*args, **kwargs)
    LOGGER.debug(f'Created temporary Docker volume {volume.short_id}')
    try:
        yield volume
    finally:
        volume.remove(force=True)
        LOGGER.debug(f'Deleted temporary Docker volume {volume.short_id}')


@contextmanager
def create_temporary_docker_container(client, *args, gpu: bool = False, **kwargs):
    if gpu:
        kwargs = {**kwargs, **{'device_requests': [DeviceRequest(count=-1, capabilities=[['gpu']])]}}
    container = client.containers.create(*args, **kwargs)
    LOGGER.debug(f'Created temporary Docker container {container.short_id}')
    try:
        yield container
    finally:
        container.remove(force=True)
        LOGGER.debug(f'Deleted temporary Docker container {container.short_id}')


def add_text_file_to_tarfile(filename: str, text: str, tar: TarFile) -> None:
    with closing(BytesIO(text.encode('utf-8'))) as f:
        tarinfo = TarInfo(filename)
        tarinfo.size = len(f.getvalue())
        tarinfo.mtime = time()
        tar.addfile(tarinfo, fileobj=f)


def add_text_file_to_container(container, filename: str, text: str) -> None:
    with TemporaryFile('w+b') as tf:
        with tarfile.open(fileobj=tf, mode='w') as tar:
            add_text_file_to_tarfile(filename, text, tar)
        tf.seek(0)
        container.put_archive(path='/', data=tf)


def extract_text_file_from_container(container, filename: str) -> str:
    with TemporaryFile('w+b') as tf:
        bits, _ = container.get_archive(filename)
        for chunk in bits:
            tf.write(chunk)
        tf.seek(0)
        text = tf.read().decode('utf-8')
